package cn.wise.sc.pay.api.association.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.association.bean.Article;
import cn.wise.sc.pay.domain.association.query.ArticleQuery;
import cn.wise.sc.pay.domain.association.vo.ArticleVO;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Map;

/**
 * 文章
 */
@FeignClient(value = "saas-v1-association",path = "/association/article",fallback = ArticleServiceImpl.class)
public interface IArticleService {

    /**
     * 发表图文分享
     *
     * @param articleQuery 文章请求实体
     * @return
     */
    @PostMapping("addArticle")
    BaseResponse addArticle(@RequestBody ArticleQuery articleQuery);

    /**
     * 上传图片或视频
     *
     * @param file 图片/视频
     * @return
     */
    @PostMapping("uploadPic")
    BaseResponse<Map<String, String>> uploadPic(@RequestBody MultipartFile file);

    /**
     * 修改文章/图文审核状态
     *
     * @param ids    文章id,多个用逗号分割
     * @param status 文章状态
     * @return
     */
    @PutMapping("updateStatus")
    BaseResponse updateStatus(@RequestParam("ids") String ids,@RequestParam("status")  byte status);

    /**
     * 批量删除图文 或 批量推荐图文
     *
     * @param ids    文章id,多个用逗号分割
     * @param status 状态
     * @return
     */
    @DeleteMapping("deleteArticles")
    BaseResponse deleteArticles(@RequestParam("ids")String ids, @RequestParam("status")Integer status);

    /**
     * 修改浏览量
     *
     * @param id      文章id
     * @param viewNum 浏览量
     * @return
     */
    @PutMapping("updateViewNum")
    BaseResponse updateViewNum(@RequestParam("id")Long id, @RequestParam("viewNum")Long viewNum);

    /**
     * 图文/文章 浏览量+1
     *
     * @param id 图文id
     * @return
     */
    @PutMapping("viewNumUp")
    BaseResponse updateViewNum(@RequestParam("id")Long id);

    /**
     * 点赞图文/文章
     *
     * @param articleId 图文id
     * @param userId    用户id
     * @return
     */
    @PutMapping("praiseNumUp")
    BaseResponse updatePraiseNum(@RequestParam("articleId")Long articleId, @RequestParam("userId")Long userId);

    /**
     * 判断用户对图文/文章已点赞
     *
     * @param articleId 图文id
     * @param userId    用户id
     * @return
     */
    @GetMapping("judgePraise")
    BaseResponse judgePraise(@RequestParam("articleId")Long articleId,@RequestParam("userId") Long userId);

    /**
     * 取消点赞
     *
     * @param id     图文id
     * @param userId 用户id
     * @return
     */
    @PutMapping("deletePraise")
    BaseResponse deletePraise(@RequestParam("id")Long id,@RequestParam("userId") Long userId);

    /**
     * 移除推荐
     *
     * @param id 图文id
     * @return
     */
    @PutMapping("deleteRecommend")
    BaseResponse deleteRecommend(@RequestParam("id")Long id);

    /**
     * 推荐
     *
     * @param id 图文id
     * @return
     */
    @PutMapping("recommend")
    BaseResponse rcommend(@RequestParam("id")Long id);

    /**
     * 收藏
     *
     * @param articleId 图文id
     * @param userId    用户id
     * @return
     */
    @PostMapping("collection")
    BaseResponse collection(@RequestParam("articleId")Long articleId, @RequestParam("userId")Long userId);

    /**
     * 判断用户对图文/文章已收藏
     *
     * @param articleId 图文id
     * @param userId    用户id
     * @return
     */
    @GetMapping("judgeCollection")
    BaseResponse judgeCollection(@RequestParam("articleId")Long articleId,@RequestParam("userId") Long userId);

    /**
     * 取消收藏
     *
     * @param id     图文id
     * @param userId 用户id
     * @return
     */
    @PutMapping("deleteCollection")
    BaseResponse deleteCollection(@RequestParam("id")Long id,@RequestParam("userId") Long userId);

    /**
     * 根据id获取图文详情
     *
     * @param id 文章id
     * @return
     */
    @GetMapping("selectById")
    BaseResponse<ArticleVO> selectArticleById(@RequestParam("id")Long id);

    /**
     * 获取未审核图文列表
     *
     * @param authorName 发表人名称
     * @param start
     * @param end
     * @return
     */
    @GetMapping("getArticleNotVerify")
    BaseResponse<Page<Article>> getArticleNotVerify(@RequestParam("authorName")String authorName,
                                                      @RequestParam("start")long start,@RequestParam("end")long end,
                                                      @RequestParam("pageNo") Integer pageNo,@RequestParam("pageSize") Integer pageSize);

    /**
     * 获取审核通过的图文列表
     *
     * @param type      搜索文章分类
     * @param order     评论数排序 asc升序 desc降序
     * @param name      文章内容
     * @param title     文章标题
     * @return
     */
    @GetMapping("getArticleVerify")
    BaseResponse<Page<Article>> getArticleVerify(@RequestParam("type")Integer type,@RequestParam("order") String order,
                                                   @RequestParam("name")String name, @RequestParam("title")String title,
                                                 @RequestParam("pageNo") Integer pageNo,@RequestParam("pageSize") Integer pageSize);

    /**
     * 用户收藏图文/文章
     *
     * @param userId    用户id
     * @param pageQuery 请求实体
     * @return
     */
    @GetMapping("userCollections")
    BaseResponse<Page<Article>> getCollectionArticle(@RequestParam("userId")Long userId,
                                                     @SpringQueryMap PageQuery pageQuery);

    /**
     * 用户发布的图文/文章
     *
     * @param userId    用户id
     * @param pageQuery 分页实体
     * @return
     */
    @GetMapping("userPublishs")
    BaseResponse<Page<Article>> getUserPublishArticle(@RequestParam("userId")Long userId,
                                                      @SpringQueryMap PageQuery pageQuery);

    /**
     * 用户点赞的图文/文章
     *
     * @param userId    用户id
     * @param pageQuery 分页实体
     * @return
     */
    @GetMapping("userPraises")
    BaseResponse<Page<Article>> getUserPraiseArticle(@RequestParam("userId")Long userId,
                                                     @SpringQueryMap PageQuery pageQuery);

    /**
     * 上一篇或下一篇图文 pon=0代表上一篇，pon=1代表下一篇
     *
     * @param articleId 文章id
     * @param pon       pon=0代表上一篇，pon=1代表下一篇
     * @return
     */
    @GetMapping("ponArticle")
    BaseResponse previousOrNextArticle(@RequestParam("articleId")Long articleId,@RequestParam("pon") Integer pon);

    /**
     * 上一篇或下一篇已审核图文 pon=0代表上一篇，pon=1代表下一篇
     *
     * @param articleId 文章id
     * @param pon       pon=0代表上一篇，pon=1代表下一篇
     * @return
     */
    @GetMapping("ponVerifyArticle")
    BaseResponse previousOrNextVerifyArticle(@RequestParam("articleId")Long articleId,@RequestParam("pon") Integer pon);
}
