package cn.wise.sc.pay.business.eshopping.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.business.eshopping.mapper.EshopStoreMapper;
import cn.wise.sc.pay.business.eshopping.service.IEshopGoodsService;
import cn.wise.sc.pay.business.eshopping.service.IEshopStoreService;
import cn.wise.sc.pay.business.eshopping.utils.RedisObjConstant;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.redis.config.lock.RedissLockUtil;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopGoods;
import cn.wise.sc.pay.domain.eshopping.model.bean.EshopStore;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RAtomicLong;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 库存service
 * </p>
 *
 * @author qh
 * @since 2020-03-24
 */
@Slf4j
@Service
public class EshopStoreServiceImpl extends ServiceImpl<EshopStoreMapper, EshopStore> implements IEshopStoreService {

    final
    RedissonClient redissonClient;
    final
    IEshopGoodsService iEshopGoodsService;


    /**
     * 主要用于分页获取商品当前页的库存
     * 一般 pageSize = 10;非一般 pageSize = 50;大于50的用主线程处理 性能差
     */
    final
    ThreadPoolExecutor poolExecutor = new ThreadPoolExecutor(10,
            20, 1, TimeUnit.SECONDS,
            new LinkedBlockingQueue<>(20), r -> new Thread("thread -- store"),
            new ThreadPoolExecutor.CallerRunsPolicy()
    );

    public EshopStoreServiceImpl(RedissonClient redissonClient,
                                 IEshopGoodsService iEshopGoodsService) {
        this.iEshopGoodsService = iEshopGoodsService;
        this.redissonClient = redissonClient;

    }


    @Override
    public BaseResponse<ConcurrentHashMap<String, Long>> storesByGoodId(Long goodId) {

        if (goodId == null || goodId == 0) {
            throw new IllegalArgumentException("商品ids无效!");
        }
        //0.判断商品是否有规格 没有时 ==> {prx:goodId}  跳过下面的规格判断
        //1.获取当前商品的所有规格
        //2.拼装redis的key: {prx:goodId:spcId:skuId}
        //3.判断每个key是否命中
        //4.不命中去数据库找，添加redis，返回

        EshopGoods good = iEshopGoodsService.getById(goodId);
        if (good == null) {
            return BaseResponse.error("没找到当前商品!", EshopStore.class.getName());
        }
        //判断规格 没有规格
        if (good.getHaveSpec() == 0) {
            String key = RedisObjConstant.PRX + good.getGoodsId();
            RAtomicLong storeStr = redissonClient.getAtomicLong(key);
            ConcurrentHashMap<String, Long> result = new ConcurrentHashMap<>(1);
            //未命中
            if (storeStr.get() == 0) {
                storeStr.set(good.getQuantity());
            }
            result.put("商品_" + good.getGoodsId(), storeStr.get());
            return BaseResponse.ok(result);
        } else {  //有规格
            QueryWrapper<EshopStore> qw = new QueryWrapper<>();
            qw.eq("good_id", good.getGoodsId());
            List<EshopStore> storeList = this.list(qw);
            if (storeList == null || storeList.size() == 0) {
                return BaseResponse.error("没找到当前商品的规格!", EshopStore.class.getName());
            }
            ConcurrentHashMap<String, Long> result = new ConcurrentHashMap<>(storeList.size());
            for (EshopStore eshopStore : storeList) {
                String key = RedisObjConstant.PRX + good.getGoodsId() + "_" + eshopStore.getSpcId();
                RAtomicLong storeStr = redissonClient.getAtomicLong(key);
                //未命中
                if (storeStr.get() == 0) {
                    storeStr.set(eshopStore.getAvailableStore());
                }
                result.put("规格_" + eshopStore.getSpcId(), storeStr.get());
            }
            return BaseResponse.ok(result);
        }
    }

    @Override
    public EshopStore storeByGoodIdAndSkuId(Long goodId, Long skuId) {

        if (goodId == null || goodId <= 0 || skuId == null || skuId <= 0) {
            throw new IllegalArgumentException("商品id和skuId不能为空!");
        }

        RAtomicLong store = redissonClient.getAtomicLong(RedisObjConstant.PRX + goodId + "_" + skuId);
        //能做redis走redis 不能就走数据库
        if (store.get() != 0) {
            EshopStore eshopStore = new EshopStore();
            eshopStore.setAvailableStore((int) store.get());
            eshopStore.setGoodId(goodId);
            eshopStore.setHaveSpc(1);
            eshopStore.setSpcId(skuId);
            return eshopStore;
        } else {
            EshopStore eshopStore = getGoodsStoreByGoodsId(goodId, skuId);
            if (eshopStore != null){
                store.set(eshopStore.getAvailableStore());
            }
            return eshopStore;
        }
    }

    @Override
    public BaseResponse<EshopStore> newStore(Long goodsId, Integer store, Long skuId) {

        if (goodsId == null || goodsId == 0 || store == null || store == 0) {
            throw new IllegalArgumentException("商品id无效或者库存数量无效！");
        }
        EshopStore eshopStore = new EshopStore();
        eshopStore.setGoodId(goodsId);
        eshopStore.setHaveSpc(0);
        eshopStore.setAvailableStore(store);
        eshopStore.setVersion(999);
        eshopStore.setRealityStore(store);
        String key = RedisObjConstant.PRX + goodsId;
        if (skuId != null && skuId >= 0) {
            eshopStore.setHaveSpc(1);
            eshopStore.setSpcId(skuId);
            key = RedisObjConstant.PRX + goodsId + "_" + skuId;
        }
        boolean result = this.save(eshopStore);
        if (result) {
            //添加成功 将库存同步到redis
            RAtomicLong rAtomicLong = redissonClient.getAtomicLong(key);
            rAtomicLong.set(store);
            return BaseResponse.okMsg("添加成功!", EshopStore.class);
        } else {
            return BaseResponse.error("添加失败!", EshopStore.class.getName());
        }
    }

    @Override
    public BaseResponse<EshopStore> addStore(Long goodId, Integer quantity, Long skuId) {

        if (goodId == null || goodId <= 0 || quantity == null || quantity <= 0) {
            throw new IllegalArgumentException("商品id无效或者添加的库存数量无效！");
        }
        //找到当前库存
        EshopStore eshopStore;
        String key;
        if (skuId == null || skuId <= 0) {
            key = RedisObjConstant.PRX + goodId;
        } else {
            key = RedisObjConstant.PRX + goodId + "_" + skuId;
        }
        eshopStore = getGoodsStoreByGoodsId(goodId, skuId);

        if (eshopStore == null) {
            return BaseResponse.error("请求确定当前商品是否存在", EshopStore.class.getName());
        }
        //判断redis中是否有key
        RAtomicLong atomicLong = redissonClient.getAtomicLong(key);
        boolean keyIsExist = atomicLong.isExists();
        if (!keyIsExist) {
            atomicLong.set(eshopStore.getAvailableStore());
        }

        UpdateWrapper<EshopStore> uw = new UpdateWrapper<>();
        uw.eq("version", eshopStore.getVersion());
        uw.eq("good_id",eshopStore.getGoodId());
        //读写锁  锁当前商品redis库存  将redis库存补到数据库相同
        boolean updateSuccess = RedissLockUtil.readWriteLock(RedisObjConstant.LOCK_PRX + goodId, arg -> {
            RAtomicLong atomicLongOnce = redissonClient.getAtomicLong(key);
            long storeValue = atomicLongOnce.get();
            int store;
            try {
                store = (int) storeValue;
                //更新数据库库存
                EshopStore updateEShopStore = new EshopStore();
                BeanUtil.copyProperties(eshopStore, updateEShopStore);
                updateEShopStore.setAvailableStore(store + quantity);
                //版本号+1
                updateEShopStore.setVersion(eshopStore.getVersion() + 1);
                boolean result = this.update(updateEShopStore, uw);
                if (result) {
                    //将redis库存 +quantity
                    atomicLongOnce.set(store + quantity);
                }
            } catch (NumberFormatException e) {
                //更新数据库库存
                EshopStore updateEShopStore = new EshopStore();
                BeanUtil.copyProperties(eshopStore, updateEShopStore);
                updateEShopStore.setAvailableStore(eshopStore.getAvailableStore() + quantity);
                //版本号+1
                updateEShopStore.setVersion(eshopStore.getVersion() + 1);
                boolean result = this.update(updateEShopStore, uw);
                if (result) {
                    //将库存同步到redis
                    atomicLongOnce.set(eshopStore.getAvailableStore() + quantity);
                }
                return false;
            }
            return true;
        });

        return updateSuccess ? BaseResponse.okMsg("库存增加成功!", EshopStore.class) :
                BaseResponse.error("库存增加失败!", EshopStore.class.getName());
    }

    @Override
    public BaseResponse<EshopStore> decreaseStore(Long goodId, Integer quantity, Long skuId) {

        if (goodId == null || goodId <= 0 || quantity == null || quantity <= 0) {
            throw new IllegalArgumentException("商品id无效或者减去的库存数量无效！");
        }
        //初始化redis的key值
        String key;
        String lockKey;
        if (skuId == null || skuId <= 0) {
            key = RedisObjConstant.PRX + goodId;
            lockKey = RedisObjConstant.LOCK_PRX + goodId;
        } else {
            key = RedisObjConstant.PRX + goodId + "_" + skuId;
            lockKey = RedisObjConstant.LOCK_PRX + goodId + "_" + skuId;
        }

        //判断redis中是否有key
        RAtomicLong bucket = redissonClient.getAtomicLong(key);
        boolean keyIsExist = bucket.isExists();
        if (!keyIsExist) {
            EshopStore eshopStore = getGoodsStoreByGoodsId(goodId, skuId);
            if (eshopStore != null) {
                bucket.set(eshopStore.getAvailableStore());
            } else {
                return BaseResponse.error("请确定当前商品或规格是否存在!", EshopStore.class.getName());
            }
        }

        //写锁 和 addStore()防止脏读
        Long store = RedissLockUtil.readWriteLock(lockKey, arg -> {
            boolean flag = true;
            long timelyStore = -1;
            //操作redis减库存
            while (flag) {
                timelyStore = bucket.get() - quantity;
                flag = !bucket.compareAndSet(bucket.get(), timelyStore);
            }

            return timelyStore;
        });

        //<0 表示库存不足
        if (store < 0) {
            //写锁 和 addStore()防止脏读
            RedissLockUtil.readWriteLock(lockKey, arg -> {
                //库存回退 此次操作失败
                bucket.getAndAdd(quantity);
                return null;
            });
            return BaseResponse.error("库存不足!", EshopStore.class.getName());
        }
        //=0 表示买完了
        if (store == 0) {
            boolean flag = true;
            //同步数据库库存
            while (flag) {
                BaseResponse<EshopStore> baseResponse = this.saveDB(goodId, skuId);
                flag = baseResponse.code == 200;
            }
        }

        return BaseResponse.okMsg("库存扣减成功!", EshopStore.class);
    }

    @Override
    public BaseResponse<EshopStore> saveDB(Long goodId, Long skuId) {


        if (goodId == null || goodId == 0) {
            throw new IllegalArgumentException("商品id,无效！");
        }
        //初始化key
        String key;
        if (skuId == null || skuId <= 0) {
            key = RedisObjConstant.PRX + goodId;
        } else {
            key = RedisObjConstant.PRX + goodId + "_" + skuId;
        }
        EshopStore eshopStore = getGoodsStoreByGoodsId(goodId, skuId);
        //获取当前商品信息
        EshopGoods goods = iEshopGoodsService.getById(goodId);
        if (goods == null || eshopStore == null) {
            log.error("没找到当前商品  =====  商品id:{}", goodId);
            throw new NullPointerException();
        }

        //获取redis中的剩余库存
        RAtomicLong atomicLong = redissonClient.getAtomicLong(key);
        boolean keyIsExist = atomicLong.isExists();
        if (keyIsExist) {
            long store = atomicLong.get();

            if (eshopStore.getAvailableStore() == 0 || (store == eshopStore.getAvailableStore())) {
                return BaseResponse.okMsg("同步成功!", EshopStore.class);
            }
            //根据提供的版本号更新
            EshopStore updateEShopStore = new EshopStore();
            BeanUtil.copyProperties(eshopStore, updateEShopStore);
            updateEShopStore.setAvailableStore((int) store);
            updateEShopStore.setUpdateTime(System.currentTimeMillis());
            updateEShopStore.setVersion(eshopStore.getVersion() + 1);
            UpdateWrapper<EshopStore> uw = new UpdateWrapper<>();
            uw.eq("version", eshopStore.getVersion());
            uw.eq("good_id",eshopStore.getGoodId());
            boolean updateSuccess = this.update(updateEShopStore, uw);
            if (!updateSuccess) {
                return BaseResponse.error("同步失败!", EshopStore.class.getName());
            }

            return BaseResponse.okMsg("同步成功!", EshopStore.class);
        } else {
            redissonClient.getAtomicLong(key).set(eshopStore.getAvailableStore());
            return BaseResponse.okMsg("同步成功!", EshopStore.class);
        }

    }

    /**
     * 根据商品id获取库存
     *
     * @param id 商品id
     * @return EshopStore
     */
    private EshopStore getGoodsStoreByGoodsId(Long id, Long skuId) {

        if (id == null || id == 0) {
            throw new IllegalArgumentException("商品id无效!");
        }
        QueryWrapper<EshopStore> qw = new QueryWrapper<>();
        qw.eq("good_id", id);
        if (skuId != null && skuId >= 0) {
            qw.eq("spc_id", skuId);
            qw.eq("have_spc", 1);
        }
        return this.getOne(qw);
    }

}
