package cn.wise.sc.pay.api.exhibition.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.TimeBucket;
import cn.wise.sc.pay.domain.exhibition.bean.EntranceQrcode;
import cn.wise.sc.pay.domain.exhibition.query.ExhibitionQuery;
import cn.wise.sc.pay.domain.exhibition.query.QRCodeGiftQuery;
import cn.wise.sc.pay.domain.exhibition.vo.RQCodeInfoVo;
import cn.wise.sc.pay.domain.exhibition.vo.TicketVO;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.cloud.openfeign.SpringQueryMap;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * @description: 展览对外暴露的服务
 * @author: qh
 * @create: 2019-12-26 15:07
 **/
@FeignClient(value = "saas-v1-exhibition", path = "/exhibition", fallback = ExhibitionsServiceImpl.class)
public interface IExhibitionsService {

    /**
     * 更新票据库存信息
     *
     * @param ticketVOS 票据集合
     * @return
     */
    @PutMapping("/store/tickets")
    BaseResponse updateTicketStore(@RequestBody List<TicketVO> ticketVOS);

    /**
     * 获取当前展览下的票据库存信息
     *
     * @param id 展览id
     * @return
     */
    @GetMapping("/store/exhibition/{id}")
    BaseResponse getExhibitionStore(@PathVariable("id") Long id);


    /**
     * 展览入场二维码 核销入场
     *
     * @param enQCodeJSON 二维码获取到的数据JSON
     * @return BaseResponse
     */
    @PutMapping("/QRCode")
    BaseResponse entranceCheck(@RequestParam("enQCodeJSON") String enQCodeJSON);

    /**
     * 根据订单号  获取订单入场二维码
     *
     * @param orderNo 订单号
     * @return BaseResponse
     */
    @GetMapping("/QRCode")
    BaseResponse getQRCode(@RequestParam("orderNo") String orderNo);

    /**
     * 添加展览
     *
     * @param exhibitionQuery
     * @return
     */
    @PostMapping("/new")
    BaseResponse addExhibition(@RequestBody ExhibitionQuery exhibitionQuery);

    /**
     * 根据id删除展览
     *
     * @param id
     * @return
     */
    @DeleteMapping("{id}")
    BaseResponse delExhibition(@PathVariable("id") Integer id);

    /**
     * 编辑展览
     *
     * @param exhibitionQuery
     * @return
     */
    @PutMapping
    BaseResponse editExhibition(@RequestBody ExhibitionQuery exhibitionQuery);

    /**
     * 上传图片
     *
     * @param file
     * @return
     */
    @PostMapping("/upload")
    BaseResponse upload(@RequestBody MultipartFile file);

    /**
     * 上新/推荐
     *
     * @param id
     * @param type
     * @return
     */
    @PutMapping("/{id}/{type}")
    BaseResponse editShowType(@PathVariable("id") long id, @PathVariable("type") int type);

    /**
     * 根据条件获取展览集合
     *
     * @param type       类型
     * @param showType   状态
     * @param timeBucket 时间段
     * @return
     */
    @GetMapping("/list")
    BaseResponse getExhibitionList(@RequestParam("type") String type, @RequestParam("showType") Integer showType,
                                   @SpringQueryMap TimeBucket timeBucket);

    /**
     * 获取广告位集合
     *
     * @return
     */
    @GetMapping("/advertising/list")
    BaseResponse getAdvertising();

    @PostMapping("/page")
    BaseResponse getExhibitionPage(@RequestBody PageWrapperQuery pageQuery);

    /**
     * 获取展览详情 根据id
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    BaseResponse getExhibitionDetail(@PathVariable("id") long id);

    /**
     * 获取用户票据信息
     *
     * @param userId 用户id
     * @return
     */
    @GetMapping("/QRCode/list/{userId}")
    @ApiOperation("获取用户的所有票据信息")
    BaseResponse<List<RQCodeInfoVo>> getAllQRCode(@PathVariable("userId") String userId);


    /**
     * 票务赠与
     *
     * @param qrCodeGiftQuery 票据信息 {@link QRCodeGiftQuery}
     * @return
     */
    @PutMapping("/gift")
    @ApiOperation("票务赠与")
    BaseResponse<EntranceQrcode> shareQRCode(@RequestBody QRCodeGiftQuery qrCodeGiftQuery);
}
