package cn.wise.sc.pay.api.order.service;

import cn.wise.sc.pay.api.order.service.impl.IOrderImpl;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.order.excel.OrderExcel;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.StatisticsGoods;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.netflix.hystrix.contrib.javanica.annotation.HystrixCommand;
import com.netflix.hystrix.contrib.javanica.annotation.HystrixProperty;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;


/**
 * <p>
 * 点单实体 服务类
 * </p>
 *
 * @author qh
 * @since 2019-12-12
 */
@FeignClient(value = "saas-v1-order", path = "/order", fallback = IOrderImpl.class)
public interface IOrdersService {

    /**
     * 创建订单
     *
     * @param orderDetailQuery
     * @return
     */
    @HystrixCommand(threadPoolKey = "payOrder", threadPoolProperties = {
            @HystrixProperty(name = "coreSize", value = "100"),
            @HystrixProperty(name = "maxQueueSize", value = "200")},
            commandProperties = {
                    @HystrixProperty(name = "circuitBreaker.errorThresholdPercentage", value = "70"),
                    @HystrixProperty(name = "circuitBreaker.sleepWindowInMilliseconds", value = "40")
            })
    @PostMapping
    OrderResponse createOrder(@RequestBody OrderDetailQuery orderDetailQuery);

    /**
     * 获取订单集合 复杂查询
     *
     * @param pageWrapperQuery
     * @return
     */
    @PostMapping("/list/complex")
    OrderResponse getOrders(@RequestBody PageWrapperQuery pageWrapperQuery);


    /**
     * 导出订单信息Excel
     *
     * @param shopId
     * @param start
     * @param end
     */
    @GetMapping("/excel/orderInfo/{shopId}")
    BaseResponse<List<OrderExcel>> orderInfoExcelByShop(@PathVariable(value = "shopId") Long shopId,
                                                        @RequestParam("start") Long start,
                                                        @RequestParam("end") Long end);

    /**
     * 取消订单
     *
     * @param orderNo 订单号
     * @return
     */
    @PutMapping("{orderNo}")
    OrderResponse cancelOrder(@PathVariable("orderNo") String orderNo);

    /**
     * 删除订单
     *
     * @param orderNo       订单号
     * @param conditionJson 条件  等于条件 Map<数据库字段,值>.toJSON()
     * @return
     */
    @DeleteMapping
    OrderResponse delOrder(@RequestParam("orderNo") String orderNo, @RequestParam("condition") String conditionJson);

    /**
     * 修改订单状态
     *
     * @param orderNo       订单号
     * @param orderStatus   订单状态
     * @param conditionJson 条件  等于条件 Map<数据库字段,值>.toJSON()
     * @return
     */
    @PutMapping("/status")
    OrderResponse changeOrderStatus(@RequestParam("orderNo") String orderNo,
                                    @RequestBody OrderStatus orderStatus,
                                    @RequestParam("conditionJson") String conditionJson);

    /**
     * 修改订单状态 同时更新订单id
     *
     * @param orderNo
     * @param orderStatus
     * @param conditionJson
     * @return
     */
    @PutMapping("/status/{userId}")
    OrderResponse changeOrderStatusByUserId(@RequestParam("orderNo") String orderNo,
                                            @PathVariable("userId") String userId,
                                            @RequestBody OrderStatus orderStatus,
                                            @RequestParam("condition") String conditionJson);

    /**
     * 修改订单
     *
     * @param orderDetailQuery order_no 必填
     * @return
     */
    @PutMapping
    OrderResponse editOrder(@RequestBody OrderDetailQuery orderDetailQuery);

    /**
     * 获取预下单号
     *
     * @return
     */
    @GetMapping("/preOrderNo")
    BaseResponse<String> getOrderNo();

    /**
     * 获取订单列表 根据用户id
     *
     * @param userId 用户id
     * @param status 订单状态
     * @return
     */
    @PostMapping("/list")
    OrderResponse getOrdersByUserId(@RequestParam("condition") String userId, @RequestBody OrderStatus status);

    /**
     * 根据条件获取订单列表
     * 简单的等于查询   如:user_id = 1  ==> map.put("user_id",1).toJSON();
     *
     * @param conditionJson Map<数据库字段,值>.toJSON()
     * @return
     */
    @GetMapping("/list")
    OrderResponse getOrders(@RequestParam("condition") String conditionJson);

    /**
     * 根据条件查询一条数据
     *
     * @param conditionJson 条件  等于条件 Map<数据库字段,值>.tostring()
     * @return
     */
    @GetMapping
    OrderResponse getOneOrder(@RequestParam("condition") String conditionJson);

    /**
     * 满足条件的条数
     *
     * @param start    起始时间
     * @param end      结束时间
     * @param status   订单状态
     * @param sellerId 商户id
     * @return
     */
    @GetMapping("/count")
    BaseResponse<Long> getCount(@RequestParam("start") Long start, @RequestParam("end") Long end,
                                @RequestBody OrderStatus status, @RequestParam("sellerId") String sellerId);

    /**
     * 分页查询
     *
     * @param pageWrapperQuery 分页条件
     * @return
     */
    @PostMapping("/page")
    OrderResponse page(@RequestBody PageWrapperQuery pageWrapperQuery);

    /**
     * 批量更新 更具orderNo
     *
     * @param ordersCancels orderNo必填
     * @return
     */
    @PutMapping("/batch")
    BaseResponse<Boolean> updateBatch(@RequestBody List<OrderDetailQuery> ordersCancels);

    /**
     * 统计  根据交易类型统计
     *
     * @param strategy 策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天
     * @param shopname 店铺名  模糊匹配
     * @param start    起始时间
     * @param end      结束时间
     * @return
     */
    @GetMapping("/statistics/status")
    BaseResponse statisticsByStatus(@RequestParam("strategy") int strategy, @RequestParam("shopname") String shopname,
                                    @RequestParam("start") long start, @RequestParam("end") long end);

    /**
     * 统计  根据交易类型统计  (折线图)
     *
     * @param strategy 策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天
     * @param shopname 店铺名  模糊匹配
     * @param start    起始时间
     * @param end      结束时间
     * @return
     */
    @GetMapping("/statistics/chart")
    BaseResponse statisticsChart(@RequestParam("strategy") int strategy, @RequestParam("shopname") String shopname,
                                 @RequestParam("start") long start, @RequestParam("end") long end);

    /**
     * 统计  根据支付类型 统计金额
     *
     * @param strategy 策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天
     * @param shopname 店铺名  模糊匹配
     * @param start    起始时间
     * @param end      结束时间
     * @return
     */
    @GetMapping("/statistics/payMethod/Money")
    BaseResponse statisticsMoneyByPatMethod(@RequestParam("strategy") int strategy, @RequestParam("shopname") String shopname,
                                            @RequestParam("start") long start, @RequestParam("end") long end);

    /**
     * 统计  根据支付类型统计笔数
     *
     * @param strategy 策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天
     * @param shopname 店铺名  模糊匹配
     * @param start    起始时间
     * @param end      结束时间
     * @return
     */
    @GetMapping("/statistics/payMethod/Count")
    BaseResponse statisticsCountByPatMethod(@RequestParam("strategy") int strategy, @RequestParam("shopname") String shopname,
                                            @RequestParam("start") long start, @RequestParam("end") long end);

    /**
     * 统计  当前统计下的所有订单
     *
     * @param strategy 策略  1:每月统计  2:每季度统计 3:每年统计 4:自定义统计 其他默认:每天
     * @param shopname 店铺名  模糊匹配
     * @param start    起始时间
     * @param end      结束时间
     * @param payment  支付方式  具体看 {@link cn.wise.sc.pay.domain.order.model.PaymentName}
     * @return
     */
    @GetMapping("/statistics/list")
    BaseResponse statisticsList(@RequestParam("strategy") int strategy, @RequestParam("shopname") String shopname,
                                @RequestParam("start") long start, @RequestParam("end") long end,
                                @RequestParam("payment") int payment);

    /**
     * 获取时间端的聚合支付金额&现金支付的总数
     *
     * @param pageWrapperQuery 查询条件构造 {@link PageWrapperQuery}
     * @return
     */
    @PostMapping("/statistics/aggregate")
    BaseResponse<HashMap<String, BigDecimal>> statisticsAggregate(@RequestBody PageWrapperQuery pageWrapperQuery);

    /**
     * 统计商品售卖的金额  通过卖家id和商品分组
     *
     * @param pageWrapperQuery 条件
     * @return
     */
    @PostMapping("/statistics/goods/market")
    BaseResponse<List<StatisticsGoods>> statisticsGoodsMarketMoneyGroupSellerGood(PageWrapperQuery pageWrapperQuery);


}
