package cn.wise.sc.pay.business.order.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.business.order.service.IBillsService;
import cn.wise.sc.pay.domain.order.excel.BillsExcel;
import cn.wise.sc.pay.domain.order.model.Bills;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @description: 账单Excel监听器
 * 官网原话:   有个很重要的点 DemoDataListener 不能被spring管理，要每次读取excel都要new,然后里面用到spring可以构造方法传进去
 * @author: qh
 * @create: 2019-12-24 15:56
 **/
public class BillsDataListener extends AnalysisEventListener<BillsExcel> {

    private static final Logger LOGGER = LoggerFactory.getLogger(BillsDataListener.class);
    /**
     * 每隔5条存储数据库，实际使用中可以3000条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 5;
    List<BillsExcel> list = new ArrayList<BillsExcel>();

    /**
     * 假设这个是一个DAO，当然有业务逻辑这个也可以是一个service。当然如果不用存储这个对象没用。
     */
    private IBillsService iBillsService;

    /**
     * 如果使用了spring,请使用这个构造方法。每次创建Listener的时候需要把spring管理的类传进来
     *
     * @param iBillsService
     */
    public BillsDataListener(IBillsService iBillsService) {
        this.iBillsService = iBillsService;
    }

    /**
     * 这个每一条数据解析都会来调用
     *
     * @param data    one row value. Is is same as {@link AnalysisContext#readRowHolder()}
     * @param context
     */
    @Override
    public void invoke(BillsExcel data, AnalysisContext context) {
        LOGGER.info("解析到一条数据:{}", JSON.toJSONString(data));
        list.add(data);
        // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (list.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理 list
            list.clear();
        }
    }

    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveData();
        LOGGER.info("所有数据解析完成！");
    }

    /**
     * 加上存储数据库
     */
    private void saveData() {
        LOGGER.info("{}条数据，开始存储数据库！", list.size());

        ObjectMapper objectMapper = new ObjectMapper();

        //转换Excel对象到Bean
        List<Bills> bills = list.stream().map(arg -> billsExcel2Bills(arg)).collect(Collectors.toList());

        iBillsService.saveBatch(bills);

        LOGGER.info("存储数据库成功！");
    }

    /**
     * 账单Excel对象转Bean对象
     *
     * @param billsExcel
     * @return
     */
    private Bills billsExcel2Bills(BillsExcel billsExcel) {

        Bills bills = new Bills();
        BeanUtil.copyProperties(billsExcel, bills);
        bills.setTransactionTime(string2Long(billsExcel.getTransactionDate(), billsExcel.getTransactionTime()));
        return bills;
    }

    /**
     * 招行Excel的日期格式:2019-12-24 15:22:21 ,需要转换成Long存数据库
     *
     * @param date 日期
     * @param time 时间
     * @return
     */
    private  Long string2Long(String date, String time) {

        String newDate = date + " " + time;
        return convertTimeToLong(newDate);
    }
    public  Long convertTimeToLong(String time) {
        Assert.notNull(time, "time is null");
        DateTimeFormatter ftf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime parse = LocalDateTime.parse(time, ftf);
        return LocalDateTime.from(parse).atZone(ZoneId.systemDefault()).toInstant().toEpochMilli();
    }
}
