package cn.wise.sc.pay.task.timer.order.handler;

import cn.wise.sc.pay.api.order.service.IOrdersAutoSettingService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.OrdersAutoSetting;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxl.job.core.biz.model.ReturnT;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * @description: 订单定时基类
 * @author: qh
 * @create: 2020-04-02 10:01
 **/
@Component
public class BaseOrderSettingHandler {

    /**
     * 一天有多少毫秒
     */
    private static final Long DAY_MILLISECONDS = 24 * 60 * 60 * 1000L;
    /**
     * 一次性处理订单的大小
     */
    private static final int BATCH_COUNT = 200;

    private IOrdersService iOrdersService;
    private IOrdersAutoSettingService iOrdersAutoSettingService;

    public BaseOrderSettingHandler(IOrdersService iOrdersService, IOrdersAutoSettingService iOrdersAutoSettingService) {
        this.iOrdersService = iOrdersService;
        this.iOrdersAutoSettingService = iOrdersAutoSettingService;
    }

    public ReturnT<String> baseExecute(ThreadPoolExecutor threadPoolExecutor,
                                       PageWrapperQuery pageWrapperQuery,
                                       Integer expire, OrderStatus orderStatus) {

        Page<Orders> page1 = getPage(0, pageWrapperQuery);
        //没取到
        if (page1 == null || page1.getTotal() == 0L) {
            return ReturnT.SUCCESS;
        } else {
            long total = page1.getTotal();
            int count = getCount(total);
            //当都更新完时执行
            CountDownLatch countDownLatch = new CountDownLatch(count);
            //第一次分页
            updateOrdersStatusCancel(page1, expire, orderStatus);
            countDownLatch.countDown();

            //第二次开始开多线程
            for (int i = 1; i < count; i++) {
                Page<Orders> page = getPage(i, pageWrapperQuery);
                threadPoolExecutor.execute(() -> {
                    if (page != null) {
                        updateOrdersStatusCancel(page1, expire, orderStatus);
                    }
                    countDownLatch.countDown();
                });
            }

            //最多等待10s 否则失败
            try {
                countDownLatch.await(20, TimeUnit.SECONDS);
            } catch (InterruptedException e) {
                e.printStackTrace();
                return ReturnT.FAIL;
            }
        }
        return ReturnT.SUCCESS;
    }

    /**
     * 获取到底取第多少次
     *
     * @param total 总次数
     * @return 次数
     */
    private int getCount(Long total) {
        return (new BigDecimal(total).divide(new BigDecimal(BATCH_COUNT), 0, RoundingMode.UP)).intValue();
    }

    /**
     * 分批次获取未支付订单
     *
     * @param pageNo 起始页
     * @return Page<Orders>
     */
    private Page<Orders> getPage(int pageNo, PageWrapperQuery pageWrapperQuery) {

        PageQuery pageQuery = new PageQuery();
        pageQuery.setPageNo(pageNo);
        pageQuery.setPageSize(BaseOrderSettingHandler.BATCH_COUNT);
        pageWrapperQuery.setPageQuery(pageQuery);
        OrderResponse response = iOrdersService.page(pageWrapperQuery);

        return response.getPage();
    }

    /**
     * 更新订单状态为取消
     *
     * @param page        订单分页信息
     * @param expire      过期时间
     * @param orderStatus 订单状态
     */
    private void updateOrdersStatusCancel(Page<Orders> page, Integer expire, OrderStatus orderStatus) {
        //第一次
        List<Orders> waitHandList = page.getRecords();
        if (waitHandList == null || waitHandList.size() == 0) {
            return;
        }
        List<OrderDetailQuery> waitUpdateList = new ArrayList<>();
        waitHandList.forEach(arg -> {
            //大于当前时间取消订单 --> 加入待更新订单
            if (arg.getUpdateTime() + DAY_MILLISECONDS * expire < System.currentTimeMillis()) {
                OrderDetailQuery orderDetailQuery = new OrderDetailQuery<>();
                orderDetailQuery.setStatus(orderStatus.name());
                orderDetailQuery.setOrderNo(arg.getOrderNo());
                orderDetailQuery.setId(arg.getId());
                waitUpdateList.add(orderDetailQuery);
            }
        });

        BaseResponse response = iOrdersService.updateBatch(waitUpdateList);
        if (response.getCode() == 200 && OrderStatus.COMPLETE.name().equals(waitHandList.get(0).getOrderStatus())){

        }
    }

    /**
     * 获取系统的自动配置
     *
     * @return OrdersAutoSetting
     */
    protected OrdersAutoSetting getOrderAutoSetting() {
        //获取订单自动取消时间
        BaseResponse<OrdersAutoSetting> response = iOrdersAutoSettingService.getOrderSetting();
        if (response.getCode() != BATCH_COUNT) {
            return null;
        }
        return response.getData();
    }
}
